"""
Setup test configuration.

Setup test config
"""

from collections import namedtuple

from irbt import Robot

import pytest

from tests.cloud_mock import CloudMock

Credential = namedtuple('Credential', 'username password')


@pytest.fixture(scope='module')
def credential():
    """Return credential."""
    return Credential('test@example.com', 'test_password')


@pytest.fixture(scope='module')
def Cloud():  # noqa: N802
    """Cloud class (mock)."""
    return CloudMock


@pytest.fixture(scope='module')
def cloud(credential):
    """Cloud instance (mock)."""
    return CloudMock(credential.username, credential.password)


@pytest.fixture(scope='module')
def robot(cloud):
    """Robot instance (mock)."""
    return Robot(cloud)



"""
Test robot class.

Early stage
"""

from irbt import Robot


def test_robot_rooms(cloud):
    """
    Test robot rooms.

    Test robot rooms
    """
    robot = Robot(cloud)
    assert(robot is not None)
    assert(robot.rooms() == [
        {'id': '1', 'name': 'Living Room', 'region_type': 'living_room'},
        {'id': '2', 'name': 'Kitchen', 'region_type': 'kitchen'},
        {'id': '3', 'name': 'Foyer 1', 'region_type': 'foyer'},
        {'id': '4', 'name': 'BedRoom 1', 'region_type': 'bedroom'},
        {'id': '5', 'name': 'BedRoom 2', 'region_type': 'bedroom'},
        {'id': '6', 'name': 'BathRoom', 'region_type': 'bathroom'},
        {'id': '7', 'name': 'Custom', 'region_type': 'custom'}
    ])


def test_robot_maps(robot):
    """
    Test robot maps.

    Test robot maps
    """
    assert(robot.maps() == [{
        'active_pmapv_details': {
            'active_pmapv': {
                'child_seg_xfer_seq_err': 0,
                'create_time': 1570282549,
                'creator': 'user',
                'last_user_pmapv_id': '134043T209849',
                'last_user_ts': 1570282549,
                'pmap_id': 'en12a9_lTglkpPqazxDWED',
                'pmapv_id': '134043T209849',
                'proc_state': 'OK_Processed'
            },
            'map_header': {
                'create_time': 1570282549,
                'id': 'en12a9_lTglkpPqazxDWED',
                'learning_percentage': 100,
                'name': 'Appartement',
                'resolution': 0.10500000417232513,
                'user_orientation_rad': 0.0,
                'version': '134043T209849'
            },
            'regions': [
                {
                    'id': '1',
                    'name': 'Living Room',
                    'region_type': 'living_room'
                },
                {
                    'id': '2',
                    'name': 'Kitchen',
                    'region_type': 'kitchen'
                },
                {
                    'id': '3',
                    'name': 'Foyer 1',
                    'region_type': 'foyer'
                },
                {
                    'id': '4',
                    'name': 'BedRoom 1',
                    'region_type': 'bedroom'
                },
                {
                    'id': '5',
                    'name': 'BedRoom 2',
                    'region_type': 'bedroom'
                },
                {
                    'id': '6',
                    'name': 'BathRoom',
                    'region_type': 'bathroom'
                },
                {
                    'id': '7',
                    'name': 'Custom',
                    'region_type': 'custom'
                }
            ]
        },
        'active_pmapv_id': '134043T209849',
        'create_time': 1570276098,
        'last_pmapv_ts': 1571613652,
        'merged_pmap_ids': [],
        'pmap_id': 'en12a9_lTglkpPqazxDWED',
        'robot_pmapv_id': '191020T232046',
        'state': 'active',
        'user_pmapv_id': '134043T209849',
        'visible': True
    }])


def test_robot_missions(robot):
    """
    Test robot missions.

    Test robot missions
    """
    assert(robot.missions() == [{
        'chrgM': 0,
        'chrgs': 0,
        'cmd': {
            'command': 'dock',
            'initiator': 'rmtApp',
            'ordered': 0,
            'state': 'desired'
        },
        'dirt': 0,
        'dockedAtStart': True,
        'done': 'stuck',
        'doneM': 0,
        'done_raw': 'stuck',
        'durationM': 0,
        'eDock': 0,
        'evacs': 0,
        'flags': 0,
        'initiator': 'ifttt',
        'nMssn': 183,
        'pauseId': 0,
        'pauseM': 0,
        'pmaps': [{'en12a9_lTglkpPqazxDWED': '191020T232046'}],
        'pmaps_robot': [{'en12a9_lTglkpPqazxDWED': '191020T232046'}],
        'robot_id': '1234ABCD1234ABCD1234ABCD1234ABCD',
        'runM': 0,
        'saves': 0,
        'softwareVer': 'lewis+3.0.11+lewis-release-rt319+14',
        'sqft': 0,
        'startEndWlBars': [4, 4],
        'startTime': 1571613647,
        'timestamp': 1571613647,
        'wifiChannel': 4,
        'wlBars': [100, 0, 0, 0, 0]
    }, {
        'chrgM': 0,
        'chrgs': 0,
        'cmd': {
            'command': 'dock',
            'initiator': 'rmtApp',
            'ordered': 0,
            'state': 'desired'
        },
        'dirt': 0,
        'dockedAtStart': True,
        'done': 'ok',
        'doneM': 0,
        'done_raw': 'ok',
        'durationM': 3,
        'eDock': 1,
        'evacs': 0,
        'flags': 0,
        'initiator': 'rmtApp',
        'nMssn': 181,
        'pauseId': 0,
        'pauseM': 0,
        'pmaps': [{'en12a9_lTglkpPqazxDWED': '191020T095313'}],
        'pmaps_robot': [{'en12a9_lTglkpPqazxDWED': '191020T095313'}],
        'robot_id': '1234ABCD1234ABCD1234ABCD1234ABCD',
        'runM': 3,
        'saves': 0,
        'softwareVer': 'lewis+3.0.11+lewis-release-rt319+14',
        'sqft': 14,
        'startEndWlBars': [4, 4],
        'startTime': 1571565013,
        'timestamp': 1571565194,
        'wifiChannel': 4,
        'wlBars': [28, 51, 21, 0, 0]
    }])


def test_robot_evac_history(robot):
    """
    Test robot evac_history.

    Test robot evac_history
    """
    assert(robot.evac_history() == {
        'EvacReports': [],
        'LifetimeData': []
    })


def test_robot_timeline(robot):
    """
    Test robot timeline.

    Test robot timeline
    """
    assert(robot.timeline() == {
        'events': [
            {
                'hkc_sns_recv_ts': 1570983779,
                'event_type': 'HKC',
                'event_id': 'f0b41b2c1d254a75b456529c7f188004',
                'event_object_id': '52e8dfca54324d83bd92be37d1523385',
                'start_time': 1570902045,
                'expires': 1573575783,
                'hkc_created_ts': 1570983781,
                'end_time': 1602438045,
                'robot_id': '1234ABCD1234ABCD1234ABCD1234ABCD',
                'event_state': 0,
                'event_details': {
                    'details_content': {
                        'headline_text_id':
                        'notification_center_nps_title_dmc',
                        'category_title_id': 'feedback_dmc',
                        'responses': [{
                            'survey_id':
                            '19ac6322-447f-42dc-89fc-b84b58b1be83',
                            'response_type': 'Survey'
                        }],
                        'action_payload': [{
                            'numberOfSchedulesToDisplay': None,
                            'video_url': None,
                            'content_text_id':
                            'notification_center_nps_desc_dmc',
                            'internal_image_url': None,
                            'display_order': 0,
                            'video_id': None,
                            'content_html_id': None,
                            'html': None, 'content_html': None,
                            'image_id': None,
                            'internal_video_url': None,
                            'image_url': None,
                            'content_id': None,
                            'image': None,
                            'content_text': 'On a scale of 0 to 10 how likely '
                                            'would you be to recommend your '
                                            'iRobot product to a friend?'
                        }],
                        'category_title_text': 'Feedback',
                        'display_type': 'separate_view',
                        'action_type': 'content',
                        'category_icon_id':
                        'notification_center_icon_feedback',
                        'headline_text': 'Got a minute to give us some '
                        'feedback on your iRobot product?'
                    },
                    'details_content_id': 15,
                    'details_content_version': 1,
                    'details_type': 'NpsSurvey'
                }
            },
            {
                'hkc_sns_recv_ts': 1570983879,
                'event_type': 'HKC',
                'event_id': '6973ba11c979476cac8266fba43af2b9',
                'event_object_id': 'f25b3fecb4354a6cba82cab122e4b2d2',
                'start_time': 1570902080,
                'expires': 1573575879,
                'hkc_created_ts': 1570983879,
                'end_time': 1602438080,
                'robot_id': '1234ABCD1234ABCD1234ABCD1234ABCD',
                'event_state': 0,
                'event_details': {
                    'details_content': {
                        'headline_text_id':
                        'notification_center_nps_title_dmc',
                        'category_title_id': 'feedback_dmc',
                        'responses': [{
                            'survey_id':
                            '97de6426-2c5f-448d-8a6a-2f99de674be7',
                            'response_type': 'Survey'
                        }],
                        'action_payload': [{
                            'numberOfSchedulesToDisplay': None,
                            'video_url': None,
                            'content_text_id':
                            'notification_center_nps_desc_dmc',
                            'internal_image_url': None,
                            'display_order': 0,
                            'video_id': None,
                            'content_html_id': None,
                            'html': None,
                            'content_html': None,
                            'image_id': None,
                            'internal_video_url': None,
                            'image_url': None,
                            'content_id': None,
                            'image': None,
                            'content_text': 'On a scale of 0 to 10 how likely '
                            'would you be to recommend your iRobot '
                            'product to a friend?'
                        }],
                        'category_title_text': 'Feedback',
                        'display_type': 'separate_view',
                        'action_type': 'content',
                        'category_icon_id':
                        'notification_center_icon_feedback',
                        'headline_text':
                        'Got a minute to give us some '
                        'feedback on your iRobot product?'
                    },
                    'details_content_id': 15,
                    'details_content_version': 1,
                    'details_type': 'NpsSurvey'
                }
            }
        ]
    })


def test_robot_vector_map(robot):
    """
    Test robot vector_map.

    Test robot vector_map
    """
    assert(robot.vector_map() == {})



"""
Mock cloud class.

Mock cloud class
"""
import pprint

responses = {
    'v1/user/associations/robots': {
        '1234ABCD1234ABCD1234ABCD1234ABCD': {
            'cap': {
                '5ghz': 1,
                'area': 1,
                'binFullDetect': 1,
                'dockComm': 1,
                'eco': 1,
                'edge': 0,
                'log': 2,
                'maps': 3,
                'multiPass': 2,
                'ota': 2,
                'pmaps': 1,
                'pose': 1,
                'pp': 0,
                'prov': 3,
                'sched': 1,
                'svcConf': 1,
                'team': 1,
                'wDevLoc': 2
            },
            'name': 'Lemon',
            'password': ':1:1234567891:abcdefghijklmnop',
            'sku': 'i715840',
            'softwareVer': 'lewis+3.0.11+lewis-release-rt319+14'
        }
    },
    'v1/1234ABCD1234ABCD1234ABCD1234ABCD/pmaps': [
        {
            'active_pmapv_details': {
                'active_pmapv': {
                    'child_seg_xfer_seq_err': 0,
                    'create_time': 1570282549,
                    'creator': 'user',
                    'last_user_pmapv_id': '134043T209849',
                    'last_user_ts': 1570282549,
                    'pmap_id': 'en12a9_lTglkpPqazxDWED',
                    'pmapv_id': '134043T209849',
                    'proc_state': 'OK_Processed'
                },
                'map_header': {
                    'create_time': 1570282549,
                    'id': 'en12a9_lTglkpPqazxDWED',
                    'learning_percentage': 100,
                    'name': 'Appartement',
                    'resolution': 0.10500000417232513,
                    'user_orientation_rad': 0.0,
                    'version': '134043T209849'
                },
                'regions': [
                    {
                        'id': '1',
                        'name': 'Living Room',
                        'region_type': 'living_room'
                    },
                    {
                        'id': '2',
                        'name': 'Kitchen',
                        'region_type': 'kitchen'
                    },
                    {
                        'id': '3',
                        'name': 'Foyer 1',
                        'region_type': 'foyer'
                    },
                    {
                        'id': '4',
                        'name': 'BedRoom 1',
                        'region_type': 'bedroom'
                    },
                    {
                        'id': '5',
                        'name': 'BedRoom 2',
                        'region_type': 'bedroom'
                    },
                    {
                        'id': '6',
                        'name': 'BathRoom',
                        'region_type': 'bathroom'
                    },
                    {
                        'id': '7',
                        'name': 'Custom',
                        'region_type': 'custom'
                    }
                ]},
            'active_pmapv_id': '134043T209849',
            'create_time': 1570276098,
            'last_pmapv_ts': 1571613652,
            'merged_pmap_ids': [],
            'pmap_id': 'en12a9_lTglkpPqazxDWED',
            'robot_pmapv_id': '191020T232046',
            'state': 'active',
            'user_pmapv_id': '134043T209849',
            'visible': True
        }
    ],
    'v1/1234ABCD1234ABCD1234ABCD1234ABCD/missionhistory': [
        {
            'chrgM': 0,
            'chrgs': 0,
            'cmd': {
                'command': 'dock',
                'initiator': 'rmtApp',
                'ordered': 0,
                'state': 'desired'
            },
            'dirt': 0,
            'dockedAtStart': True,
            'done': 'stuck',
            'doneM': 0,
            'done_raw': 'stuck',
            'durationM': 0,
            'eDock': 0,
            'evacs': 0,
            'flags': 0,
            'initiator': 'ifttt',
            'nMssn': 183,
            'pauseId': 0,
            'pauseM': 0,
            'pmaps': [{
                    'en12a9_lTglkpPqazxDWED': '191020T232046'
            }],
            'pmaps_robot': [{
                'en12a9_lTglkpPqazxDWED': '191020T232046'
            }],
            'robot_id': '1234ABCD1234ABCD1234ABCD1234ABCD',
            'runM': 0,
            'saves': 0,
            'softwareVer': 'lewis+3.0.11+lewis-release-rt319+14',
            'sqft': 0,
            'startEndWlBars': [
                4,
                4
            ],
            'startTime': 1571613647,
            'timestamp': 1571613647,
            'wifiChannel': 4,
            'wlBars': [
                100,
                0,
                0,
                0,
                0
            ]
        },
        {
            'chrgM': 0,
            'chrgs': 0,
            'cmd': {
                'command': 'dock',
                'initiator': 'rmtApp',
                'ordered': 0,
                'state': 'desired'
            },
            'dirt': 0,
            'dockedAtStart': True,
            'done': 'ok',
            'doneM': 0,
            'done_raw': 'ok',
            'durationM': 3,
            'eDock': 1,
            'evacs': 0,
            'flags': 0,
            'initiator': 'rmtApp',
            'nMssn': 181,
            'pauseId': 0,
            'pauseM': 0,
            'pmaps': [{
                'en12a9_lTglkpPqazxDWED': '191020T095313'
            }],
            'pmaps_robot': [{
                'en12a9_lTglkpPqazxDWED': '191020T095313'
            }],
            'robot_id': '1234ABCD1234ABCD1234ABCD1234ABCD',
            'runM': 3,
            'saves': 0,
            'softwareVer': 'lewis+3.0.11+lewis-release-rt319+14',
            'sqft': 14,
            'startEndWlBars': [
                4,
                4
            ],
            'startTime': 1571565013,
            'timestamp': 1571565194,
            'wifiChannel': 4,
            'wlBars': [
                28,
                51,
                21,
                0,
                0
            ]
        }
    ],
    'v1/evachistory': {
        'EvacReports': [],
        'LifetimeData': []
    },
    'v1/robots/1234ABCD1234ABCD1234ABCD1234ABCD/timeline': {
        'events': [
            {
                'hkc_sns_recv_ts': 1570983779,
                'event_type': 'HKC',
                'event_id': 'f0b41b2c1d254a75b456529c7f188004',
                'event_object_id': '52e8dfca54324d83bd92be37d1523385',
                'start_time': 1570902045,
                'expires': 1573575783,
                'hkc_created_ts': 1570983781,
                'end_time': 1602438045,
                'robot_id': '1234ABCD1234ABCD1234ABCD1234ABCD',
                'event_state': 0,
                'event_details': {
                    'details_content': {
                        'headline_text_id':
                        'notification_center_nps_title_dmc',
                        'category_title_id': 'feedback_dmc',
                        'responses': [{
                            'survey_id':
                            '19ac6322-447f-42dc-89fc-b84b58b1be83',
                            'response_type': 'Survey'
                        }],
                        'action_payload': [{
                            'numberOfSchedulesToDisplay': None,
                            'video_url': None,
                            'content_text_id':
                            'notification_center_nps_desc_dmc',
                            'internal_image_url': None,
                            'display_order': 0,
                            'video_id': None,
                            'content_html_id': None,
                            'html': None, 'content_html': None,
                            'image_id': None,
                            'internal_video_url': None,
                            'image_url': None,
                            'content_id': None,
                            'image': None,
                            'content_text': 'On a scale of 0 to 10 how likely '
                                            'would you be to recommend your '
                                            'iRobot product to a friend?'
                        }],
                        'category_title_text': 'Feedback',
                        'display_type': 'separate_view',
                        'action_type': 'content',
                        'category_icon_id':
                        'notification_center_icon_feedback',
                        'headline_text': 'Got a minute to give us some '
                        'feedback on your iRobot product?'
                    },
                    'details_content_id': 15,
                    'details_content_version': 1,
                    'details_type': 'NpsSurvey'
                }
            },
            {
                'hkc_sns_recv_ts': 1570983879,
                'event_type': 'HKC',
                'event_id': '6973ba11c979476cac8266fba43af2b9',
                'event_object_id': 'f25b3fecb4354a6cba82cab122e4b2d2',
                'start_time': 1570902080,
                'expires': 1573575879,
                'hkc_created_ts': 1570983879,
                'end_time': 1602438080,
                'robot_id': '1234ABCD1234ABCD1234ABCD1234ABCD',
                'event_state': 0,
                'event_details': {
                    'details_content': {
                        'headline_text_id':
                        'notification_center_nps_title_dmc',
                        'category_title_id': 'feedback_dmc',
                        'responses': [{
                            'survey_id':
                            '97de6426-2c5f-448d-8a6a-2f99de674be7',
                            'response_type': 'Survey'
                        }],
                        'action_payload': [{
                            'numberOfSchedulesToDisplay': None,
                            'video_url': None,
                            'content_text_id':
                            'notification_center_nps_desc_dmc',
                            'internal_image_url': None,
                            'display_order': 0,
                            'video_id': None,
                            'content_html_id': None,
                            'html': None,
                            'content_html': None,
                            'image_id': None,
                            'internal_video_url': None,
                            'image_url': None,
                            'content_id': None,
                            'image': None,
                            'content_text': 'On a scale of 0 to 10 how likely '
                            'would you be to recommend your iRobot product to '
                            'a friend?'
                        }],
                        'category_title_text': 'Feedback',
                        'display_type': 'separate_view',
                        'action_type': 'content',
                        'category_icon_id':
                        'notification_center_icon_feedback',
                        'headline_text': 'Got a minute to give us some '
                        'feedback on your iRobot product?'
                    },
                    'details_content_id': 15,
                    'details_content_version': 1,
                    'details_type': 'NpsSurvey'
                }
            }
        ]
    },
    'v1/1234ABCD1234ABCD1234ABCD1234ABCD/pmaps/en12a9_lTglkpPqazxDWED/versions'
    '/134043T209849/umf': {

    }
}


class CloudMock:
    """
    Cloud class.

    It allow interactions with the irbt cloud api
    """

    api = None

    class Api:
        """
        Api class.

        HTTP cloud api
        """

        cloud = None

        def __init__(self, cloud):
            """Initialize cloud mock."""
            self.cloud = cloud

        def get(self, *a, params=None, as_json=True):
            """
            HTTP GET.

            Authenticated GET method
            """
            parts = [
                # self.cloud._disc_api['httpBaseAuth'],
                self.cloud._api_version
            ]
            parts.extend(a)
            pprint.pprint(parts)
            return responses['/'.join(parts)]

    def __init__(self, username=None, password=None):
        """
        Initialize usefull params.

        Use user credentials to call the login method during the init
        if provided.
        """
        self._disc_api = {
            'httpBaseAuth': 'https://api.example.com'
        }
        self._api_version = 'v1'
        self.service_url = None
        self._api_key = None
        self._aws_host = None
        self._aws_region = None
        self._http_base = None
        self.app_id = 'IOS-'
        self.auth = None
        self.mqtt_endpoint = None
        self.mqtt_topic = None
        self.debug_mqtt = None
        self.access_key_id = None
        self.secret_key = None
        self.session_token = None
        if username and password:
            self.login(username=username, password=password)
        self.api = CloudMock.Api(self)

    # login to the irbt cloud
    def login(self, username, password):
        """Mock login."""
        pass

    def robots(self):
        """
        Return a list of robots provided by the api.

        The first json key is the robot id
        """
        return self.api.get('user', 'associations', 'robots')



"""
Cloud class for authentication and connexion setup.

It allow interactions with the irbt cloud api
"""
import json
from urllib.parse import urlparse

from aws_requests_auth.aws_auth import AWSRequestsAuth

import requests

from .logger import logging

logger = logging.getLogger(__name__)


class Cloud:
    """
    Cloud class.

    It allow interactions with the irbt cloud api
    """

    api = None

    class Api:
        """
        Api class.

        HTTP cloud api
        """

        cloud = None

        def __init__(self, cloud):
            """Init our cloud."""
            self.cloud = cloud

        def get(self, *a, params=None, as_json=True):
            """
            HTTP GET.

            Authenticated GET method
            """
            parts = [
                self.cloud._disc_api['httpBaseAuth'],
                self.cloud._api_version
            ]
            parts.extend(a)
            # pprint.pprint('/'.join(parts))
            response = requests.get('/'.join(parts),
                                    params=params,
                                    auth=self.cloud.auth)
            if response.status_code != 200:
                if response.status_code == 403:
                    logger.error('Authentication failed, provided aws '
                                 'credentials are unauthorized for the '
                                 'requested endpoint: %s',
                                 '/'.join(parts))
                    exit(1)
                else:
                    raise Exception('CloudAPIGetError<{}>'.format(
                        response.status_code))
            if as_json:
                # pprint.pprint(response.json())
                return response.json()
            return response

    def __init__(self, username=None, password=None):
        """
        Initialize usefull params.

        Use user credentials to call the login method during the init
        if provided.
        """
        self._disc_api = None
        self._api_version = 'v1'
        self.service_url = None
        self._api_key = None
        self._aws_host = None
        self._aws_region = None
        self._http_base = None
        self.app_id = 'ANDROID-5B4E9B96-C1A8-48BE-ACD7-B41C9F3DC1DE'
        self.auth = None
        self.mqtt_endpoint = None
        self.mqtt_topic = None
        self.debug_mqtt = None
        self.access_key_id = None
        self.secret_key = None
        self.session_token = None
        self.shadow_client = None
        self.device = None

        if username and password:
            self.login(username=username, password=password)
        self.api = Cloud.Api(self)
    # discover the current api and mqtt params

    def _disc(self):
        params = {
            'country_code': 'FR'
        }
        self._disc_api = json.loads(
            requests.get(
                'https://disc-prod.iot.irobotapi.com/v1/app/discover',
                params=params)
            .content.decode('utf-8'))

        # check if all the needed keys are there
        api_keys = ['gigya', 'httpBaseAuth', 'httpBase', 'awsRegion',
                    'mqtt', 'irbtTopics']
        for api_key in api_keys:
            if api_key not in self._disc_api.keys():
                logger.error('discover url not working, %s key is not there',
                             api_key)
                exit(1)
        if 'api_key' not in self._disc_api['gigya'].keys():
            logger.error('discover url is not working, api key is not there')
            exit(1)

        self._api_key = self._disc_api['gigya']['api_key']  # gigya api key

        # the aws api service url
        # used for aws api gateway request
        self.service_url = '%s/%s/' % (
            self._disc_api['httpBaseAuth'], self._api_version)

        parsed_uri = urlparse(self.service_url)
        self._aws_host = '{uri.netloc}'.format(
            uri=parsed_uri)  # host used for iam requests
        self._aws_region = self._disc_api['awsRegion']  # aws region
        self._http_base = self._disc_api['httpBase']  # irbt unauth api
        self.mqtt_endpoint = self._disc_api['mqtt']  # the mqtt endpoint
        # the topic used to send commands
        # broken for now (fixme)
        self.mqtt_topic = self._disc_api['irbtTopics']

    # gigya login gather credentials used later in ibrbt login api
    def _gigya_login(self, username, password):
        gigya_login_data = {
            'ApiKey': self._api_key,
            'ctag': 'webbridge',
                    'format': 'json',
                    'loginID': username,
                    'password': password,
                    'sessionExpiration': '-2',
                    # if not set we gather cookies instead of gigya secrets
                    'targetEnv': 'mobile'
        }
        return json.loads(
            requests.post(
                # default login  url for gigya :
                # https://developers.gigya.com/display/GD/accounts.linkAccounts+REST
                'https://accounts.us1.gigya.com/accounts.login',
                data=gigya_login_data)
            .content.decode('utf-8'))

    # loggin in irbt login api to gather aws credentials
    def _irbt_login_api(self, signature, timestamp, uid):
        data = {
            'assume_robot_ownership': '0',
            'signature': signature,
            'timestamp': timestamp,
            'uid': uid
        }
        # retrieve the aws credentials
        unauth_api = json.loads(
            requests.post(
                self._http_base + '/v1/login/account',
                data=json.dumps(data))
            .content.decode('utf-8'))
        return (
            unauth_api['credentials']['AccessKeyId'],
            unauth_api['credentials']['SecretKey'],
            unauth_api['credentials']['SessionToken'])

    # login to the irbt cloud
    def login(self, username, password):
        """
        Retrieve the aws credentials used by the apis.

        The login is staged, p1 we login to gigya, p2 we use p1 credentials
        in irbt login api, that finally return aws credentials to
        use in all the subsequent calls (mqtt and api gateway)
        """
        # discover cloud settings
        self._disc()

        # login to gigya (get uid, timestamp and signature)
        gigya_login = self._gigya_login(username, password)

        # get aws keys through irbt login api
        try:
            (self.access_key_id,
             self.secret_key,
             self.session_token) = self._irbt_login_api(
                gigya_login['UIDSignature'],
                gigya_login['signatureTimestamp'],
                gigya_login['UID'])
        except KeyError:
            logger.error('Authentication failed, wrong login/password')
            exit(1)
        # use AWSRequestsAuth module to set the aws authentication headers
        # (Signature Version 4 Signing Process)
        self.auth = AWSRequestsAuth(
            aws_access_key=self.access_key_id,
            aws_secret_access_key=self.secret_key,
            aws_token=self.session_token,
            aws_host=self._aws_host,
            aws_region=self._aws_region,
            aws_service='execute-api')

    # Assoc
    def assoc(self, password, robot_id):
        """
        Associate provided robotid and password to the account.

        It only needs to be done once
        """
        data = {'password': password}
        return requests.post(
            self.service_url + 'user/associations/robots/%s?app_id=%s'
            % (robot_id, self.app_id),
            auth=self.auth, json=data).content

    # return a list of our robots
    def robots(self):
        """
        Return a list of robots provided by the api.

        The first json key is the robot id
        """
        return self.api.get('user', 'associations', 'robots')



"""
Robot class.

It allows interactions with the robot cloud api
"""
import functools
import json

from AWSIoTPythonSDK.MQTTLib import AWSIoTMQTTShadowClient

from .logger import logging

logger = logging.getLogger(__name__)


def _output_status(payload, response_status, token):
    if not payload:
        return
    payload_dict = json.loads(payload)
    if not payload_dict:
        return

    prefix = payload_dict['state']['reported']
    infos = {
        'Battery': '%s%%' % prefix['batPct'],
        'Name': prefix['name'],
        'Bin Present': prefix['bin']['present'],
        'Bin full': prefix['bin']['full'],
        'Mission phase': prefix['cleanMissionStatus']['cycle'],
        'Initiator': prefix['cleanMissionStatus']['initiator'],
        'Last Command': prefix['lastCommand']['command'],
        # 'Last initiator': prefix['lastCommand']['initiator'],
        'Time Zone': prefix['timezone'],
        'Cloud env': prefix['cloudEnv'],
        'Cloud connected': prefix['connected'],
        'Country': prefix['country'],
        'Wlan mac address': prefix['hwPartsRev']['wlan0HwAddr']
    }
    logger.info(json.dumps(infos))


class Robot:
    """
    Class Robot.

    Used to interact with the robot, it needs a cloud instance
    """

    command = None

    class Commands:
        """
        The robot mqtt command class.

        Used for the mqtt commands
        """

        start: None
        stop: None
        pause: None
        dock: None
        status: None
        find: None

        def __init__(self, robot):
            """
            Declare the partial functools.

            Used to gate what command are sent through mqtt
            """
            self.start = functools.partial(robot._cmd, 'start')
            self.stop = functools.partial(robot._cmd, 'stop')
            self.pause = functools.partial(robot._cmd, 'pause')
            self.dock = functools.partial(robot._cmd, 'dock')
            self.status = functools.partial(robot._cmd, 'status')
            self.find = functools.partial(robot._cmd, 'find')

    def __init__(self, cloud=None, rid=None, output_raw=None):
        """
        Initialize the robot instance.

        Check if a cloud is provided and
        set the current map id.
        """
        self.command = Robot.Commands(self)
        # if no cloud connexion is passed,create one using provided credentials
        if not cloud:
            raise Exception('You need to provide a cloud connection')
        else:
            self._cloud = cloud

        # use provided id or first robot available
        if rid:
            self._id = rid
        else:
            self._id = list(self._cloud.robots())[0]

        self._current_map_id = None
        self._current_user_pmapv_id = None
        self.maps()
        self.device = None
        self.output_raw = output_raw
        self.name = None
        self.shadow_client = None

    def connect(self):
        """
        Instantiate mqtt clients and delete them when exiting.

        We use AWSIoTMQTTShadowClient to create our MQTT connection.
        This manager will close the connection on exit
        """
        self.shadow_client = AWSIoTMQTTShadowClient(
            self._cloud.app_id,
            useWebsocket=True)
        self.shadow_client.configureEndpoint(self._cloud.mqtt_endpoint, 443)
        self.shadow_client.configureCredentials('config/aws-root-ca1.cer')
        self.shadow_client.configureIAMCredentials(
            self._cloud.access_key_id,
            self._cloud.secret_key,
            self._cloud.session_token)
        self.shadow_client.configureAutoReconnectBackoffTime(1, 128, 20)
        self.shadow_client.configureConnectDisconnectTimeout(10)
        self.shadow_client.configureMQTTOperationTimeout(5)
        # Set keepAlive interval to be 1 second and connect
        # Raise exception if there is an error in connecting to AWS IoT

        try:
            if not self.shadow_client.connect(5):
                raise Exception('AWSIoTMQTTShadowClientCouldNotConnect')
        except ValueError as e:
            logger.error("shadow_client.connect returned '%s'"
                         ', credentials are not authorized.', str(e))
            exit(1)
        self.device = self.shadow_client.createShadowHandlerWithName(self._id,
                                                                     True)
        self.connection = self.shadow_client.getMQTTConnection()
        logger.info('[+] mqtt connected')

    def disconnect(self):
        """Disconnect the mqtt stuff."""
        logger.info('[+] mqtt disconnected')
        self.connection.disconnect()

    # return maps and set active one
    def maps(self):
        """
        Return the map lists.

        Used to return a map lists and to set the active one in the instance
        only retrieve first map for now (fixme)
        """
        params = {
            'visible': 'true',
            'activeDetails': '1'
        }
        maps = self._cloud.api.get(self._id, 'pmaps', params=params)
        path = ['active_pmapv_details', 'active_pmapv', 'pmap_id']
        self._current_map_id = maps[0][path[0]][path[1]][path[2]]
        self._current_user_pmapv_id = maps[0]['user_pmapv_id']
        return maps

    def rooms(self):
        """
        Return the room lists.

        Retrieve a correctly formated room list for humans
        """
        return (self.maps()[0]['active_pmapv_details']['regions'])

    def missions(self):
        """
        Return achieved mission list.

        return a json with the list of the finished missions and their statuses
        """
        params = {
            'filterType': 'omit_quickly_canceled_not_scheduled'
        }
        return self._cloud.api.get(self._id,
                                   'missionhistory',
                                   params=params)

    def evac_history(self):
        """
        Return logs of evacuations.

        return a json with logs of evacuations
        """
        params = {
            'robotId': self._id,
            'maxAge': 90
        }
        return self._cloud.api.get('evachistory', params=params)

    def timeline(self):
        """
        Return event timeline.

        return a json with the event timeline
        """
        params = {
            'event_type': 'HKC',
            'details_type_filter': 'all'
        }
        return self._cloud.api.get('robots',
                                   self._id,
                                   'timeline',
                                   params=params)

    def vector_map(self, map_id=None, user_pmapv_id=None):
        """
        Return a map as json.

        Return a json with coordinates of the map
        and history of the mission if any.
        """
        if not map_id:
            map_id = self._current_map_id
        if not user_pmapv_id:
            user_pmapv_id = self._current_user_pmapv_id
        return self._cloud.api.get(self._id,
                                   'pmaps',
                                   map_id,
                                   'versions',
                                   user_pmapv_id,
                                   'umf')

    def _make_payload(self, room_ids, cmd):
        payload = {
            'state': 'desired',
            'command': cmd,
            'initiator': 'rmtApp',
            'ordered': 0,
        }
        if cmd == 'start' and room_ids:
            logger.info('starting cleaning of %s', room_ids)
            regions = [{'region_id': room_id}
                       for room_id in room_ids.split(',')] \
                if ',' in room_ids else [{'region_id': room_ids}]
            payload.update({
                'pmap_id': self._current_map_id,
                'regions': regions,
                'user_pmapv_id': self._current_user_pmapv_id
            })
        return payload

    def _cmd(self, cmd, room_ids=None, print_output=_output_status):
        topic = '%s/things/%s/cmd' % (self._cloud.mqtt_topic, self._id)
        qos = 1

        payload = self._make_payload(room_ids, cmd)

        if cmd == 'status':
            self.device.shadowGet(print_output, 5)
            self.device.shadowRegisterDeltaCallback(print_output)
            return 0  # exit(0)
        logger.info(
            'executing command %s on robot %s', cmd, self._id)
        if self.connection.publish(topic, json.dumps(payload), qos):
            self.device.shadowGet(print_output, 5)
            self.device.shadowRegisterDeltaCallback(print_output)
        else:
            raise Exception('MqttPublish%sError' % cmd)



"""
This is a library to interact with iRbt cloud.

Use it to retrieve informations and command your iRbt appliances.
"""

from .cloud import Cloud  # noqa: F401
from .logger import enable_mqtt_logging, logging  # noqa: F401
from .parse_command_line import get_argument_parser  # noqa: F401
from .robot import Robot  # noqa: F401



#!/usr/bin/env python3
"""
CLI for roomba.

Set IRBT_LOGIN and IRBT_PASSWORD environment variables
"""

import json
import os
import sys

from irbt import (Cloud, Robot, enable_mqtt_logging, get_argument_parser,
                  logging)


def print_output(payload, response_status, token):
    """
    Print callback for the mqtt stuff.

    Used if none is provided
    """
    if not payload:
        return
    payload_dict = json.loads(payload)
    if not payload_dict:
        return
    print(json.dumps(payload_dict['state']['reported']))


logger = logging.getLogger('irbt-cli')

parser = get_argument_parser()
if len(sys.argv) == 1:
    parser.print_help(sys.stderr)
    exit(1)
args = parser.parse_args()

# enable mqtt debug
if args.debug_mqtt:
    enable_mqtt_logging()

# instance Robot
username = os.environ.get('IRBT_LOGIN')
password = os.environ.get('IRBT_PASSWORD')

if not username or not password:
    logger.error('IRBT_LOGIN or IRBT_PASSWORD are not set')
    exit(1)

# cloud connexion
cloud = Cloud(username=username, password=password)

# associate robot
if args.associate:
    if not args.robot_id or not args.robot_password:
        logger.error('No robot ID \
                      or password provided for the association')
        exit(1)
    else:
        cloud.assoc(args.robot_password, args.robot_id)

try:
    robot_id = args.robot_id if args.robot_id else list(cloud.robots())[0]
except IndexError:
    logger.error('[-] No robot found in the account')
    exit(0)

# show robot list
if args.list_robots:
    for index, robot in enumerate(cloud.robots()):
        print('robot {} : {}'.format(index, robot))
    sys.exit(0)
# show password of selected robot
elif args.show_password:
    print('Robot password: {}'.format(cloud.robots()[robot_id]['password']))
    sys.exit(0)
# show robot general infos
elif args.robot_infos:
    infos = cloud.robots()[robot_id]
    if args.output_json:
        print('%s' % json.dumps(infos))
    else:
        print("""Informations on {robot_id}
    Name: {name}
    Model: {sku}
    Password: {password}
    Software Version: {software_ver}""".format(
            robot_id=robot_id,
            name=infos['name'],
            sku=infos['sku'],
            password=infos['password'],
            software_ver=infos['softwareVer']))
        sys.exit(0)

# instance robot !
robot = Robot(rid=args.robot_id, cloud=cloud, output_raw=args.output_raw)

# list rooms
if args.list_rooms:
    for room in robot.rooms():
        print('{}: {}'.format(room['name'], room['id']))

# contains all the map (parse me)
elif args.missions:
    print(json.dumps(robot.missions()))
# evacutation history
elif args.evac_history:
    print(json.dumps(robot.evac_history()))
# timeline
elif args.timeline:
    print(json.dumps(robot.timeline()))
# get active map
elif args.map:
    print(json.dumps(robot.vector_map()))
# commands
elif args.cmd:
    robot.connect()
    getattr(robot.command, args.cmd)(room_ids=args.room_ids,
                                     print_output=print_output)
    robot.disconnect()
sys.exit(0)



"""
Setup python package.

Setup python package.
"""

import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='irbt',
    version_config={
        "version_format": "{tag}.dev{sha}",
        "starting_version": "0.3.0"
    },
    author='Cyril Leclerc',
    author_email='cyril.leclerc@gmail.com',
    description='A Library to interact with irbt appliances',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/tidalf/irbt',
    packages=setuptools.find_packages(),
    scripts=['bin/irbt-cli.py'],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
    setup_requires=['better-setuptools-git-version'],
)



"""
Test cloud class.

Early stage
"""


def test_cloud(credential, Cloud):  # noqa: N803
    """
    Test cloud.

    Test cloud
    """
    assert(credential.username == 'test@example.com')
    assert(credential.password == 'test_password')
    cloud = Cloud(credential.username, credential.password)
    assert(cloud)



"""
Parse command line args.

Helper for our cli
"""

import argparse

args_dict = [
    {
        'flag': '-m',
        'name': '--map',
        'action': 'store_true',
        'required': False,
        'dest': 'map',
        'help': 'output current map',
        'default': False
    },
    {
        'flag': '-c',
        'name': '--cmd',
        'action': 'store',
        'required': False,
        'nargs': '?',
        'const': 1,
        'dest': 'cmd',
        'help': 'cmd for the robot (start, stop, dock, pause, find)',
        'default': False
    },
    {
        'flag': '-d',
        'name': '--debug-mqtt',
        'action': 'store_true',
        'required': False,
        'dest': 'debug_mqtt',
        'help': 'debug-mqtt',
        'default': False
    },
    {
        'flag': '-M',
        'name': '--missions',
        'action': 'store_true',
        'required': False,
        'dest': 'missions',
        'help': 'Missions history',
        'default': False
    },
    {
        'flag': '-e',
        'name': '--evachistory',
        'action': 'store_true',
        'required': False,
        'dest': 'evac_history',
        'help': 'Evac history',
        'default': False
    },
    {
        'flag': '-t',
        'name': '--timeline',
        'action': 'store_true',
        'required': False,
        'dest': 'timeline',
        'help': 'Timeline',
        'default': False
    },
    {
        'flag': '-l',
        'name': '--list-rooms',
        'action': 'store_true',
        'required': False,
        'dest': 'list_rooms',
        'help': 'List rooms',
        'default': False
    },
    {
        'flag': '-r',
        'name': '--room-ids',
        'action': 'store',
        'required': False,
        'nargs': '?',
        'const': 1,
        'dest': 'room_ids',
        'help': 'Specify rooms id'
        'room ids to clean',
        'default': None
    },
    {
        'flag': '-p',
        'name': '--robot-password',
        'action': 'store_true',
        'required': False,
        'dest': 'show_password',
        'help': 'Show robot password',
        'default': None
    },
    {
        'flag': '-R',
        'name': '--robots',
        'action': 'store_true',
        'required': False,
        'dest': 'list_robots',
        'help': 'List robots',
        'default': None
    },
    {
        'flag': '-i',
        'name': '--robot-id',
        'action': 'store',
        'required': False,
        'nargs': '?',
        'const': 1,
        'dest': 'robot_id',
        'help': 'Specify robot id',
        'default': None
    },
    {
        'flag': '-I',
        'name': '--robot-infos',
        'action': 'store_true',
        'required': False,
        'dest': 'robot_infos',
        'help': 'Show robot infos',
        'default': None
    },
    {
        'flag': '-j',
        'name': '--output-json',
        'action': 'store_true',
        'required': False,
        'dest': 'output_json',
        'help': 'Output as Json if possible',
        'default': None
    },
    {
        'flag': '-o',
        'name': '--assoc',
        'action': 'store_true',
        'required': False,
        'dest': 'associate',
        'help': 'Associate provided robot id with account',
        'default': None
    },
    {
        'flag': '-w',
        'name': '--raw',
        'action': 'store_true',
        'required': False,
        'dest': 'output_raw',
        'help': 'Output Raw Json (from server api) if possible',
        'default': None
    },
    {
        'flag': '-P',
        'name': '--set-robot-password',
        'action': 'store',
        'required': False,
        'nargs': '?',
        'const': 1,
        'dest': 'robot_password',
        'help': 'Provide robot password for association',
        'default': None
    }
]


def get_argument_parser():
    """Parse command line args."""
    arguments_parser = argparse.ArgumentParser()

    def get_and_del(dct, name):
        value = dct[name]
        del dct[name]
        return value

    for cmd in args_dict:
        flag = get_and_del(cmd, 'flag')
        name = get_and_del(cmd, 'name')
        arguments_parser.add_argument(flag, name, **cmd)
    return arguments_parser



"""
Logger helpers functions.

Used for to set separate log levers depending of the loggers
"""
import logging
logging.basicConfig(level=logging.DEBUG)

urllib3_logger = logging.getLogger('urllib3.connectionpool')
urllib3_logger.setLevel(logging.ERROR)

mqtt_logger = logging.getLogger('AWSIoTPythonSDK.core')
mqtt_logger.setLevel(logging.ERROR)


def enable_mqtt_logging():
    """
    Enable Mqtt Logging.

    Set the mqtt logger to DEBUG when asked.
    """
    mqtt_logger.setLevel(logging.DEBUG)
    stream_handler = logging.StreamHandler()
    formatter = logging.Formatter(
        '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    stream_handler.setFormatter(formatter)
    mqtt_logger.addHandler(stream_handler)
